/*
 * Tipo: Banco
 *
 * Marca Registrada (c) 2000 CiN
 * Todos os direitos reservados.
 *
 * Este software  confidencial e propriedade intelectual do
 * CiN - Centro de Informtica.
 * Voc no deve utilizar indevidamente este produto em desacordo com o
 * contrato estabelecido com a empresa.
 */

package banco;

import conta.Conta;
import conta.SIException;

/**
 * Esta classe modela um banco, com suas contas e as operaes sobre elas.
 */
public class Banco {

    /**
     * O nmero de contas atualmente cadastradas.
     */
    private int indice = 0;

    /**
     * As contas cadastradas neste banco.
     */
    private Conta[] contas;

    /**
     * O nmero mximo de contas que podem ser cadastradas neste banco.
     */
    private final int MAX_CONTAS = 50;

    /**
     * Cria um novo banco.
     */
    public Banco () {
        contas = new Conta[MAX_CONTAS];
        indice = 0;
    }

    /**
     * Cadastra uma conta no banco. Avisa se no pode efetuar o cadastro.
     */
    public void cadastro(Conta c) throws CEException {
        if (indice < MAX_CONTAS) {
            if (procura(c.getNumero()) != null) {
                throw new CEException(c.getNumero());
            }
            contas[indice] = c;
            indice = indice + 1;
        } else {
            throw new IndexOutOfBoundsException();
        }
    }

    /**
     * Procura uma conta neste banco, dada o nmero da conta procurada.
     */
    private Conta procura(String num) {
        int i = 0; // Variveis locais ao mtodo.
        boolean achou = false;
        Conta result;

        while ((! achou) & (i < indice)) {
            if (contas[i].getNumero().equals(num))
                achou = true;
            else
                i = i + 1;
        }

        if (achou == true)
            result = contas[i];
        else
            result = null;

        return result;
    }

    /**
     * Debita o valor especificado na conta dada.
     */
    public void debito(String num, double val)  throws CNEException, SIException {
        Conta c;
        c = this.procura(num);
        if (c == null)
            throw new CNEException(num);
        else
            c.debitar(val);
    }

    /**
     * Retorna a soma dos saldos de todas as contas do banco.
     */
    public double saldoTotal() {
        int i;
        double total;

        total = 0.0;
        for (i = 0; i < indice; i = i + 1)
            total = total + contas[i].getSaldo();

        return total;
    }

    /**
     * Retorna o nmero de clientes do banco.
     */
    public int numClientes() {
        return indice;
    }

    /**
     * M&eacute;todo para testes.
     */
    public static void main(String args[]) {
        Banco b = new Banco();
        try {
            b.cadastro(new Conta("1312", 20.00));
            b.cadastro(new Conta("43125"));
            b.cadastro(new Conta("4723",100.0));
        } catch (CEException cee) {
            cee.printStackTrace();
        }
        System.out.println("O banco tem R$"+b.saldoTotal()+" em depositos");
        try {
            b.debito("13", 12.00);  // Esta linha dar um erro.
        } catch (SIException cee) {
            cee.printStackTrace();
        } catch (CNEException cee) {
            cee.printStackTrace();
        }
        System.out.println ("O banco tem "+b.numClientes()+" clientes.");
    }

}

